package cmd

import (
	"fmt"

	"github.com/golang-migrate/migrate/v4"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
)

// MigrateCmd is the "migrate" command.
type MigrateCmd struct {
	options *Options `no-flag:"t"`
}

// Execute does the actual migration.
func (cmd *MigrateCmd) Execute([]string) error {
	setMinLoggingLevel(zerolog.InfoLevel)
	m, err := database.NewMigrate(cmd.options.Dsn, migration.Source)
	if err != nil {
		return fmt.Errorf("failed to init migration engine: %s", err)
	}
	defer m.Close()

	if err := m.Up(); err != nil {
		if err == migrate.ErrNoChange {
			log.Info().Msg("The database is already up-to-date")
		} else {
			return fmt.Errorf("failed to run migration: %s", err)
		}
	} else {
		log.Info().Msg("database successfully upgraded")
	}

	return nil
}
