package tests

import (
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
)

func TestTemplateManagement(t *testing.T) {
	tester := NewAPITester(t, nil)
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Login("janedoe", "janedoe")

	_ = t.Run("Create", func(t *testing.T) {
		defer tester.SetT(t)()
		var template models.Template
		require.Equal(t, http.StatusCreated,
			tester.MustPostJSON("/api/v1/template/janedoe", JSONObj{
				"name":        "t1",
				"language":    "mustache",
				"produces":    "text/html",
				"body-format": "text",
				"body":        "<html><body>Hi {{name}}!</body></html>",
			}, &template))
		assert.Equal(t, "t1", template.Name)
		assert.Equal(t, "janedoe", template.Account)
	}) &&
		t.Run("Get", func(t *testing.T) {
			defer tester.SetT(t)()
			var template models.Template
			require.Equal(t, http.StatusOK,
				tester.MustGetJSON("/api/v1/template/janedoe/t1", &template),
			)
			assert.Equal(t, "t1", template.Name)
			assert.Equal(t, "janedoe", template.Account)
		}) &&
		t.Run("List", func(t *testing.T) {
			defer tester.SetT(t)()
			var templateList []models.Template
			require.Equal(t, http.StatusOK,
				tester.MustGetJSON("/api/v1/template/janedoe", &templateList))
			assert.Equal(t, 1, len(templateList))
			assert.Equal(t, "t1", templateList[0].Name)
		}) &&
		t.Run("Update", func(t *testing.T) {
			defer tester.SetT(t)()
			var template models.Template
			require.Equal(t, http.StatusOK,
				tester.MustPutJSON("/api/v1/template/janedoe/t1", JSONObj{
					"language":    "mustache",
					"produces":    "text/mjml",
					"body-format": "text",
					"body":        "<mjml><mj-text>hello {{name}}</mj-text></mjml>",
				}, &template))
			assert.Equal(t, "t1", template.Name)
			assert.Equal(t, "janedoe", template.Account)
			assert.Equal(t, "text/mjml", template.Produces)
		}) &&
		t.Run("Rename", func(t *testing.T) {
			defer tester.SetT(t)()
			var template models.Template
			require.Equal(t, http.StatusOK,
				tester.MustPutJSON("/api/v1/template/janedoe/t1", JSONObj{
					"name":        "t2",
					"language":    "mustache",
					"produces":    "text/html",
					"body-format": "text",
					"body":        "<html><body>Hi {{name}}!</body></html>",
				}, &template))
			assert.Equal(t, "t2", template.Name)
			assert.Equal(t, "janedoe", template.Account)

			require.Equal(t, http.StatusOK,
				tester.MustGetJSON("/api/v1/template/janedoe/t2", &template),
			)
			assert.Equal(t, "t2", template.Name)
			assert.Equal(t, "janedoe", template.Account)
		})
}
