package tests

import (
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
)

func TestUserManagement(t *testing.T) {
	tester := NewAPITester(t, nil)
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	_ = t.Run("Create", func(t *testing.T) {
		defer tester.SetT(t)()
		var info models.Userinfo
		require.Equal(t, http.StatusCreated,
			tester.MustPostJSON("/api/v1/user", JSONObj{
				"username": "janedoe",
			}, &info))
		assert.Equal(t, "janedoe", info.Username)
		assert.False(t, info.IsAdmin)
	}) &&
		t.Run(
			"Get",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var info models.Userinfo
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/user/janedoe", &info),
				)
				assert.Equal(t, "janedoe", info.Username)
				assert.False(t, info.IsAdmin)
			}) &&
		t.Run(
			"List",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var infoList []models.Userinfo
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/user", &infoList))
				assert.Equal(t, 2, len(infoList))
				assert.Contains(t, infoList, models.Userinfo{Username: "admin", IsAdmin: true})
				assert.Contains(t, infoList, models.Userinfo{Username: "janedoe", IsAdmin: false})
			})
}
