package tests

import (
	"encoding/base64"
	"fmt"
	"io/ioutil"
	"strings"
	"sync"
	"sync/atomic"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/cmd/rednerd/cmd"
	"orus.io/orus-io/rednerd/engines/wpd"
	"orus.io/orus-io/rednerd/models"
)

func TestWPDHeavyLoad(t *testing.T) {
	if testing.Short() {
		t.Skip("skipping wpd heavy load tests in short mode")
	}
	tester := NewAPITester(t, &cmd.EngineOptions{
		Wpd: wpd.EngineOptions{
			WorkerPoolSize: 10,
		},
	})
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Login("janedoe", "janedoe")

	var wg sync.WaitGroup
	var count uint64
	for i := 0; i != 1000; i++ {
		wg.Add(1)
		go func(i int) {
			defer wg.Done()
			var result []models.Document
			body := "<html><body><h1>Lorem Ipsum</h1>" + strings.Repeat("<p>"+loremIpsum+"</p>", i/10+1) + "</body></html>"
			require.Equal(t, 200, tester.MustPostJSON(
				"/api/v1/render",
				JSONObj{
					"accept": "application/pdf",
					"document": JSONObj{
						"type":        "text/html",
						"body-format": "text",
						"body":        body,
					},
				},
				&result,
			))
			require.Equal(t, 1, len(result))
			if body, err := base64.StdEncoding.DecodeString(result[0].Body); assert.NoError(t, err) && assert.NotZero(t, len(body)) {
				if i < 10 {
					require.NoError(t, ioutil.WriteFile(fmt.Sprintf("wpd-heavy-load-%.3d.pdf", i), body, 0o600))
				}
			}
			t.Logf("%d/1000: Rendered doc n°%d", atomic.AddUint64(&count, 1), i)
		}(i)
	}
	wg.Wait()
}

var loremIpsum = `Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.`
